summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorMiklos Vajna <vmiklos@collabora.co.uk>2016-08-17 12:17:19 +0200
committerMiklos Vajna <vmiklos@collabora.co.uk>2016-08-17 14:02:34 +0200
commit69fb6a307172e244497bc618a102afccdd7c93b7 (patch)
tree3517ee695fb43611f1c0e4e987bcb1cf7139a84e
parent62c4a8aacf76771e97a8da35096e6ad69a11979a (diff)
lok::Document::getCommandValues: expose redline info
Index is added as a property for each item, so that later changes can be identified by the index when they are accepted/rejected. Change-Id: I9362d208fdbed1f46d64558d44498d2b19150c81
-rw-r--r--desktop/qa/desktop_lib/test_desktop_lib.cxx25
-rw-r--r--desktop/source/lib/init.cxx53
-rw-r--r--libreofficekit/qa/gtktiledviewer/gtktiledviewer.cxx68
3 files changed, 145 insertions, 1 deletions
diff --git a/desktop/qa/desktop_lib/test_desktop_lib.cxx b/desktop/qa/desktop_lib/test_desktop_lib.cxx
index 017439160ef7..a1e7cc04df93 100644
--- a/desktop/qa/desktop_lib/test_desktop_lib.cxx
+++ b/desktop/qa/desktop_lib/test_desktop_lib.cxx
@@ -92,6 +92,7 @@ public:
void testContextMenuWriter();
void testContextMenuImpress();
void testNotificationCompression();
+ void testRedlineWriter();
CPPUNIT_TEST_SUITE(DesktopLOKTest);
CPPUNIT_TEST(testGetStyles);
@@ -119,6 +120,7 @@ public:
CPPUNIT_TEST(testContextMenuWriter);
CPPUNIT_TEST(testContextMenuImpress);
CPPUNIT_TEST(testNotificationCompression);
+ CPPUNIT_TEST(testRedlineWriter);
CPPUNIT_TEST_SUITE_END();
uno::Reference<lang::XComponent> mxComponent;
@@ -1380,6 +1382,29 @@ void DesktopLOKTest::testNotificationCompression()
CPPUNIT_ASSERT_EQUAL(std::string("1"), std::get<1>(notifs[i++]));
}
+void DesktopLOKTest::testRedlineWriter()
+{
+ // Load a Writer document, enable change recording and press a key.
+ comphelper::LibreOfficeKit::setActive();
+ LibLODocument_Impl* pDocument = loadDoc("blank_text.odt");
+ uno::Reference<beans::XPropertySet> xPropertySet(mxComponent, uno::UNO_QUERY);
+ xPropertySet->setPropertyValue("RecordChanges", uno::makeAny(true));
+ pDocument->pClass->postKeyEvent(pDocument, LOK_KEYEVENT_KEYINPUT, 't', 0);
+ pDocument->pClass->postKeyEvent(pDocument, LOK_KEYEVENT_KEYUP, 't', 0);
+
+ // Get redline info.
+ boost::property_tree::ptree aTree;
+ char* pJSON = pDocument->m_pDocumentClass->getCommandValues(pDocument, ".uno:AcceptTrackedChanges");
+ std::stringstream aStream(pJSON);
+ free(pJSON);
+ CPPUNIT_ASSERT(!aStream.str().empty());
+ boost::property_tree::read_json(aStream, aTree);
+ // Make sure that pressing a key creates exactly one redline.
+ CPPUNIT_ASSERT_EQUAL(static_cast<size_t>(1), aTree.get_child("redlines").size());
+
+ comphelper::LibreOfficeKit::setActive(false);
+}
+
CPPUNIT_TEST_SUITE_REGISTRATION(DesktopLOKTest);
CPPUNIT_PLUGIN_IMPLEMENT();
diff --git a/desktop/source/lib/init.cxx b/desktop/source/lib/init.cxx
index 0d4fb4f42f4a..17955e850197 100644
--- a/desktop/source/lib/init.cxx
+++ b/desktop/source/lib/init.cxx
@@ -50,6 +50,7 @@
#include <com/sun/star/util/URLTransformer.hpp>
#include <com/sun/star/datatransfer/clipboard/XClipboard.hpp>
#include <com/sun/star/text/TextContentAnchorType.hpp>
+#include <com/sun/star/document/XRedlinesSupplier.hpp>
#include <editeng/fontitem.hxx>
#include <editeng/flstitem.hxx>
@@ -80,6 +81,7 @@
#include <comphelper/sequence.hxx>
#include <sfx2/sfxbasemodel.hxx>
#include <svl/undo.hxx>
+#include <unotools/datetime.hxx>
#include <app.hxx>
@@ -1810,8 +1812,55 @@ static char* getUndoOrRedo(LibreOfficeKitDocument* pThis, UndoOrRedo eCommand)
return pJson;
}
+/// Returns the JSON representation of the redline stack.
+static char* getTrackedChanges(LibreOfficeKitDocument* pThis)
+{
+ LibLODocument_Impl* pDocument = static_cast<LibLODocument_Impl*>(pThis);
+
+ uno::Reference<document::XRedlinesSupplier> xRedlinesSupplier(pDocument->mxComponent, uno::UNO_QUERY);
+ if (!xRedlinesSupplier.is())
+ return nullptr;
+
+ uno::Reference<container::XEnumeration> xRedlines = xRedlinesSupplier->getRedlines()->createEnumeration();
+ boost::property_tree::ptree aRedlines;
+ for (size_t nIndex = 0; xRedlines->hasMoreElements(); ++nIndex)
+ {
+ uno::Reference<beans::XPropertySet> xRedline(xRedlines->nextElement(), uno::UNO_QUERY);
+ boost::property_tree::ptree aRedline;
+ aRedline.put("index", nIndex);
+
+ OUString sAuthor;
+ xRedline->getPropertyValue("RedlineAuthor") >>= sAuthor;
+ aRedline.put("author", sAuthor.toUtf8().getStr());
+
+ OUString sType;
+ xRedline->getPropertyValue("RedlineType") >>= sType;
+ aRedline.put("type", sType.toUtf8().getStr());
+
+ OUString sComment;
+ xRedline->getPropertyValue("RedlineComment") >>= sComment;
+ aRedline.put("comment", sComment.toUtf8().getStr());
+
+ util::DateTime aDateTime;
+ xRedline->getPropertyValue("RedlineDateTime") >>= aDateTime;
+ OUString sDateTime = utl::toISO8601(aDateTime);
+ aRedline.put("dateTime", sDateTime.toUtf8().getStr());
+
+ aRedlines.push_back(std::make_pair("", aRedline));
+ }
+
+ boost::property_tree::ptree aTree;
+ aTree.add_child("redlines", aRedlines);
+ std::stringstream aStream;
+ boost::property_tree::write_json(aStream, aTree);
+ char* pJson = strdup(aStream.str().c_str());
+ return pJson;
+}
+
static char* doc_getCommandValues(LibreOfficeKitDocument* pThis, const char* pCommand)
{
+ SolarMutexGuard aGuard;
+
OString aCommand(pCommand);
static const OString aViewRowColumnHeaders(".uno:ViewRowColumnHeaders");
static const OString aCellCursor(".uno:CellCursor");
@@ -1832,6 +1881,10 @@ static char* doc_getCommandValues(LibreOfficeKitDocument* pThis, const char* pCo
{
return getUndoOrRedo(pThis, UndoOrRedo::REDO);
}
+ else if (aCommand == ".uno:AcceptTrackedChanges")
+ {
+ return getTrackedChanges(pThis);
+ }
else if (aCommand.startsWith(aViewRowColumnHeaders))
{
ITiledRenderable* pDoc = getTiledRenderable(pThis);
diff --git a/libreofficekit/qa/gtktiledviewer/gtktiledviewer.cxx b/libreofficekit/qa/gtktiledviewer/gtktiledviewer.cxx
index a53bd00c5d7c..9351b61b0ea1 100644
--- a/libreofficekit/qa/gtktiledviewer/gtktiledviewer.cxx
+++ b/libreofficekit/qa/gtktiledviewer/gtktiledviewer.cxx
@@ -445,13 +445,73 @@ static void addMoreUnoParam(GtkWidget* /*pWidget*/, gpointer userdata)
gtk_widget_show_all(pUnoParamAreaBox);
}
+/// Exposes the info returned for tracked changes.
+static void documentRedline(GtkWidget* pButton, gpointer /*pItem*/)
+{
+ TiledWindow& rWindow = lcl_getTiledWindow(pButton);
+ LOKDocView* pDocView = LOK_DOC_VIEW(rWindow.m_pDocView);
+ // Get the data.
+ LibreOfficeKitDocument* pDocument = lok_doc_view_get_document(pDocView);
+ char* pValues = pDocument->pClass->getCommandValues(pDocument, ".uno:AcceptTrackedChanges");
+ std::stringstream aInfo;
+ aInfo << "lok::Document::getCommandValues('.uno:AcceptTrackedChanges') returned '" << pValues << "'" << std::endl;
+ g_info("%s", aInfo.str().c_str());
+ std::stringstream aStream(pValues);
+ free(pValues);
+ assert(!aStream.str().empty());
+ boost::property_tree::ptree aTree;
+ boost::property_tree::read_json(aStream, aTree);
+
+ // Create the dialog.
+ GtkWidget* pDialog = gtk_dialog_new_with_buttons("Manage Changes",
+ GTK_WINDOW (gtk_widget_get_toplevel(GTK_WIDGET(pDocView))),
+ GTK_DIALOG_MODAL,
+ "Close",
+ GTK_RESPONSE_OK,
+ nullptr);
+ GtkWidget* pContentArea = gtk_dialog_get_content_area(GTK_DIALOG (pDialog));
+
+ // Build the table.
+ GtkTreeStore* pTreeStore = gtk_tree_store_new(5, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+ for (const auto& rValue : aTree.get_child("redlines"))
+ {
+ GtkTreeIter aTreeIter;
+ gtk_tree_store_append(pTreeStore, &aTreeIter, nullptr);
+ gtk_tree_store_set(pTreeStore, &aTreeIter,
+ 0, rValue.second.get<int>("index"),
+ 1, rValue.second.get<std::string>("author").c_str(),
+ 2, rValue.second.get<std::string>("type").c_str(),
+ 3, rValue.second.get<std::string>("comment").c_str(),
+ 4, rValue.second.get<std::string>("dateTime").c_str(),
+ -1);
+ }
+ GtkWidget* pTreeView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pTreeStore));
+ std::vector<std::string> aColumns = {"Index", "Type", "Comment", "Author", "Timestamp"};
+ for (size_t nColumn = 0; nColumn < aColumns.size(); ++nColumn)
+ {
+ GtkCellRenderer* pRenderer = gtk_cell_renderer_text_new();
+ GtkTreeViewColumn* pColumn = gtk_tree_view_column_new_with_attributes(aColumns[nColumn].c_str(),
+ pRenderer,
+ "text", nColumn,
+ nullptr);
+ gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
+ }
+ gtk_box_pack_start(GTK_BOX(pContentArea), pTreeView, TRUE, TRUE, 2);
+
+ // Show the dialog.
+ gtk_widget_show_all(pDialog);
+ gtk_dialog_run(GTK_DIALOG(pDialog));
+
+ gtk_widget_destroy(pDialog);
+}
+
static void documentRepair(GtkWidget* pButton, gpointer /*pItem*/)
{
TiledWindow& rWindow = lcl_getTiledWindow(pButton);
LOKDocView* pDocView = LOK_DOC_VIEW(rWindow.m_pDocView);
// Get the data.
LibreOfficeKitDocument* pDocument = lok_doc_view_get_document(pDocView);
// How it in linear time, so first redo in reverse order, then undo.
std::vector<std::string> aTypes = {".uno:Redo", ".uno:Undo"};
std::vector<boost::property_tree::ptree> aTrees;
for (size_t nType = 0; nType < aTypes.size(); ++nType)
@@ -1410,6 +1470,12 @@ static GtkWidget* createWindow(TiledWindow& rWindow)
gtk_toolbar_insert(GTK_TOOLBAR(pUpperToolbar), pDocumentRepair, -1);
g_signal_connect(G_OBJECT(pDocumentRepair), "clicked", G_CALLBACK(documentRepair), nullptr);
+ GtkToolItem* pDocumentRedline = gtk_tool_button_new(nullptr, nullptr);
+ gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(pDocumentRedline), "system-run");
+ gtk_tool_item_set_tooltip_text(pDocumentRedline, "Document redlines");
+ gtk_toolbar_insert(GTK_TOOLBAR(pUpperToolbar), pDocumentRedline, -1);
+ g_signal_connect(G_OBJECT(pDocumentRedline), "clicked", G_CALLBACK(documentRedline), nullptr);
+
gtk_toolbar_insert(GTK_TOOLBAR(pUpperToolbar), gtk_separator_tool_item_new(), -1);
// Find.
@@ -445,13 +445,73 @@ static void addMoreUnoParam(GtkWidget* /*pWidget*/, gpointer userdata)
// Get the data.
LibreOfficeKitDocument* pDocument = lok_doc_view_get_document(pDocView);
- // How it in linear time, so first redo in reverse order, then undo.
+ // Show it in linear time, so first redo in reverse order, then undo.
std::vector<std::string> aTypes = {".uno:Redo", ".uno:Undo"};
std::vector<boost::property_tree::ptree> aTrees;